@@ -1,11 +1,11 @@
 /* hello.c -- print a greeting message and exit.
 
    Copyright (C) 1992, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
-   2005, 2006 Free Software Foundation, Inc.
+   2005, 2006, 2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -33,7 +33,6 @@ static const struct option longopts[] =
   { NULL, 0, NULL, 0 }
 };
 
-static void my_exit (void);
 static void print_help (void);
 static void print_version (void);
 
@@ -42,7 +41,7 @@ main (int argc, char *argv[])
 {
   int optc;
   int t = 0, n = 0, lose = 0;
-  char *greeting = NULL;
+  const char *greeting = NULL;
 
   program_name = argv[0];
 
@@ -55,20 +54,28 @@ main (int argc, char *argv[])
   textdomain (PACKAGE);
 #endif
 
+  /* Even exiting has subtleties.  The /dev/full device on GNU/Linux
+     can be used for testing whether writes are checked properly.  For
+     instance, hello >/dev/full should exit unsuccessfully.  On exit,
+     if any writes failed, change the exit status.  This is
+     implemented in the Gnulib module "closeout".  */
+  atexit (close_stdout);
+
   while ((optc = getopt_long (argc, argv, "g:hntv", longopts, NULL)) != -1)
     switch (optc)
       {
-      /* One goal here is having --help and --version exit immediately.  */
+      /* One goal here is having --help and --version exit immediately,
+         per GNU coding standards.  */
       case 'v':
         print_version ();
-        my_exit ();
+        exit (EXIT_SUCCESS);
         break;
       case 'g':
         greeting = optarg;
         break;
       case 'h':
         print_help ();
-        my_exit ();
+        exit (EXIT_SUCCESS);
         break;
       case 'n':
         n = 1;
@@ -85,10 +92,11 @@ main (int argc, char *argv[])
     {
       /* Print error message and exit.  */
       if (optind < argc)
-        fputs (_("Too many arguments\n"), stderr);
+        fprintf (stderr, _("%s: extra operand: %s\n"),
+		 program_name, argv[optind]);
       fprintf (stderr, _("Try `%s --help' for more information.\n"),
                program_name);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   /* Print greeting message and exit. */
@@ -115,24 +123,7 @@ main (int argc, char *argv[])
       puts (greeting);
     }
   
-  my_exit ();
-}
-
-
-
-/* Even exiting has subtleties.  The /dev/full device on GNU/Linux can
-   be used for testing whether writes are checked properly.  For instance,
-   hello >/dev/null should exit unsuccessfully.  */
-   
-static void
-my_exit (void)
-{
-  /* Exit unsuccessfully if the write failed.  This is implemented in
-     the Gnulib module "closeout".  */
-  close_stdout ();
-
-  /* Otherwise, exit successfully.  */
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
 
@@ -195,7 +186,8 @@ print_version (void)
      year comes around.  */
   printf (_("\
 Copyright (C) %s Free Software Foundation, Inc.\n\
-License: GNU GPL v2+ <http://www.gnu.org/licenses/gpl.html>\n\
-This is free software.  There is NO WARRANTY, to the extent permitted by law.\n"),
-              "2006");
+License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
+This is free software: you are free to change and redistribute it.\n\
+There is NO WARRANTY, to the extent permitted by law.\n"),
+              "2007");
 }
